==> wahcade-0.22.tbz: share/wahcade/win_cpviewer.py <==
# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        win_cpviewer.py
# Description: cp viewer window
# Copyright (c) 2007   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import os
import sys
import new
import types
import ConfigParser
try:
    import xml.etree.ElementTree as ET # in python >=2.5
except ImportError:
    try:
        import cElementTree as ET # effbot's C module
    except ImportError:
        try:
            import elementtree.ElementTree as ET # effbot's pure Python module
        except ImportError:
            try:
                import lxml.etree as ET # ElementTree API using libxml2
            except ImportError:
                import warnings
                warning.warn("could not import ElementTree "
                             "(http://effbot.org/zone/element-index.htm)")
#gtk
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import pango

#project modules
from constants import *
from wc_common import WahCade
from mamewah_ini import MameWahIni

#constants
_items = [
    'P1JoyUp', 'P1JoyDown', 'P1JoyLeft', 'P1JoyRight',
    'P1B1', 'P1B2', 'P1B3', 'P1B4', 'P1B5', 'P1B6', 'P1B7', 'P1B8',
    'P2JoyUp', 'P2JoyDown', 'P2JoyLeft', 'P2JoyRight',
    'P2B1', 'P2B2', 'P2B3', 'P2B4', 'P2B5', 'P2B6', 'P2B7', 'P2B8',
    'P1JoyType', 'P2JoyType', 'GameName', 'NumPlayers', 'History',
    'Spinner1', 'Spinner2']
_ctrls_map = {
    'P1JoyUp': ['P1_JOYSTICK_UP', 'P1_JOYSTICKLEFT_UP'],
    'P1JoyDown': ['P1_JOYSTICK_DOWN', 'P1_JOYSTICKLEFT_DOWN'],
    'P1JoyLeft': ['P1_JOYSTICK_LEFT', 'P1_JOYSTICKLEFT_LEFT'],
    'P1JoyRight': ['P1_JOYSTICK_RIGHT', 'P1_JOYSTICKLEFT_RIGHT'],
    'P1B1': ['P1_BUTTON1'],
    'P1B2': ['P1_BUTTON2'],
    'P1B3': ['P1_BUTTON3'],
    'P1B4': ['P1_BUTTON4'],
    'P1B5': ['P1_BUTTON5'],
    'P1B6': ['P1_BUTTON6'],
    'P1B7': ['P1_BUTTON7'],
    'P1B8': ['P1_BUTTON8'],
    'P2JoyUp': ['P2_JOYSTICK_UP', 'P1_JOYSTICKRIGHT_UP'],
    'P2JoyDown': ['P2_JOYSTICK_DOWN', 'P1_JOYSTICKRIGHT_DOWN'],
    'P2JoyLeft': ['P2_JOYSTICK_LEFT', 'P1_JOYSTICKRIGHT_LEFT'],
    'P2JoyRight': ['P2_JOYSTICK_RIGHT', 'P1_JOYSTICKRIGHT_RIGHT'],
    'P2B1': ['P2_BUTTON1'],
    'P2B2': ['P2_BUTTON2'],
    'P2B3': ['P2_BUTTON3'],
    'P2B4': ['P2_BUTTON4'],
    'P2B5': ['P2_BUTTON5'],
    'P2B6': ['P2_BUTTON6'],
    'P2B7': ['P2_BUTTON7'],
    'P2B8': ['P2_BUTTON8'],
    'P1JoyType': ['P1Controls'],
    'P2JoyType': ['P2Controls'],
    'GameName': ['gamename'],
    'NumPlayers': ['numPlayers'],
    'History': ['miscDetails'],
    'Spinner1': ['P1_PADDLE', 'P1_PADDLE_EXT'],
    'Spinner2': ['P1_PADDLE_V', 'P1_PADDLE_V_EXT']
}

class WinCPViewer(WahCade):
    """cpviewer based control panel viewer"""
    WinMain = None

    def init(self):
        """setup cp viewer window"""
        #build gui
        self.winCPViewer = gtk.Window()
        self.fixd = gtk.Fixed()
        self.imgBackground = gtk.Image()
        self.fixd.add(self.imgBackground)
        self.winCPViewer.add(self.fixd)
        self.imgBackground.show()
        self.fixd.show()
        self.winCPViewer.set_decorated(False)
        self.winCPViewer.set_transient_for(self.WinMain.winMain)
        self.winCPViewer.set_modal(True)
        self.winCPViewer.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.winCPViewer.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
        #self.winCPViewer.connect('destroy', self.on_winCPViewer_destroy)
        #get keyboard & mouse events
        self.winCPViewer.add_events(
            gtk.gdk.POINTER_MOTION_MASK | 
            gtk.gdk.BUTTON_PRESS_MASK | 
            gtk.gdk.BUTTON_RELEASE_MASK |
            gtk.gdk.KEY_PRESS |
            gtk.gdk.KEY_RELEASE)
        self.winCPViewer.connect('key-press-event', self.WinMain.on_winMain_key_press)
        self.winCPViewer.connect('key-release-event', self.WinMain.on_winMain_key_press)
        self.winCPViewer.connect('motion-notify-event', self.WinMain.on_winMain_key_press)
        #load layout & controls info
        self.ctrls_ini = self.get_controls_ini(self.cpviewer_ini.get('controls_ini_file'))
        self.load_layout(self.cpviewer_ini.get('viewer_layout'))
        #show window
        self.winCPViewer.show()
        if not self.WinMain.wahcade_ini.getint('show_cursor'):
            self.hide_mouse_cursor(self.winCPViewer)
        if self.WinMain.config_opts.fullscreen and not self.WinMain.config_opts.windowed:
            self.winCPViewer.fullscreen()
        self.app_number = 0

    def on_winMain_delete_event(self, *args):
        """exit window"""
        #hide window
        self.winCPViewer.hide()
        return True

    def _make_label(self, widget_name):
        """create a label (inside an event box)"""
        evb = gtk.EventBox()
        evb.set_name(widget_name)
        lbl = gtk.Label(widget_name)
        lbl.show()
        lbl.set_line_wrap(True)
        lbl.set_text('')
        evb.set_size_request(100, 25)
        evb.set_visible_window(True)
        evb.show()
        evb.add(lbl)
        evb.set_property('visible', False)
        return evb
        
    def load_layout(self, xml_filename):
        """parse cpviewer xml file and create & position labels"""
        #init widgets
        cpv_widgets = {}
        for i, widget_name in enumerate(_items):
            evb = self._make_label(widget_name)
            cpv_widgets[widget_name] = evb
            self.fixd.add(evb)
        #get category info
        for event, ctrl_element in ET.iterparse(xml_filename):
            if ctrl_element.tag in _items:
                #get label / event box
                evb = cpv_widgets[ctrl_element.tag]
                lbl = evb.child
                #font
                fd = ctrl_element.find('FontName').text
                if ctrl_element.find('FontBold').text == 'True':
                    fd += ' Bold'
                if ctrl_element.find('FontItalic').text == 'True':
                    fd += ' Italic'
                fd += ' %s' % (ctrl_element.find("FontSize").text)
                font_desc = pango.FontDescription(fd)
                #colours
                transparent = True
                if ctrl_element.find('Transparent') is not None:
                    transparent = (ctrl_element.find('Transparent').text == 'True')
                fg_col = gtk.gdk.color_parse(self.get_colour(abs(int(ctrl_element.find('ForeColor').text))))
                bg_col = gtk.gdk.color_parse(self.get_colour(abs(int(ctrl_element.find('BackColor').text))))
                lbl.modify_font(font_desc)
                lbl.modify_fg(gtk.STATE_NORMAL, fg_col)
                if transparent:
                    evb.set_visible_window(False)
                else:
                    evb.modify_bg(gtk.STATE_NORMAL, bg_col)
                #visible?
                evb.set_property('visible', ctrl_element.find('Visible').text == 'True')
                #alignment
                if ctrl_element.find('TextAlign').text == 'MiddleLeft':
                    align = 0.0
                    justify = gtk.JUSTIFY_LEFT
                elif ctrl_element.find('TextAlign').text == 'MiddleCenter':
                    align = 0.5
                    justify = gtk.JUSTIFY_CENTER
                elif ctrl_element.find('TextAlign').text == 'MiddleRight':
                    align = 1.0
                    justify = gtk.JUSTIFY_RIGHT
                lbl.set_alignment(align, 0.5)
                lbl.set_property('justify', justify)
                #move & size
                evb.set_size_request(
                    int(ctrl_element.find('Width').text),
                    int(ctrl_element.find('Height').text))
                self.fixd.move(
                    evb,
                    int(ctrl_element.find('Left').text),
                    int(ctrl_element.find('Top').text))
            elif ctrl_element.tag  == 'MainForm':
                #setup background, etc
                main_width = int(ctrl_element.find('Width').text)
                main_height = int(ctrl_element.find('Height').text)
                main_img = self.get_path(ctrl_element.find('BGImage').text)
                if ctrl_element.find('BackColor') is not None:
                    main_bg_col = gtk.gdk.color_parse(self.get_colour(abs(int(ctrl_element.find('BackColor').text))))
                    self.winCPViewer.modify_bg(gtk.STATE_NORMAL, main_bg_col)
                self.winCPViewer.set_size_request(main_width, main_height)
                self.winCPViewer.set_default_size(main_width, main_height)
                self.imgBackground.set_size_request(main_width, main_height)
                if os.path.exists(main_img):
                    self.imgBackground.set_from_file(main_img)
        #done
        ctrl_element.clear()

    def get_controls_ini(self, controls_file):
        """load controls.ini file"""
        ctrls_ini = ConfigParser.SafeConfigParser()
        ctrls_ini.read(controls_file)
        return ctrls_ini

    def display_game_details(self, rom_name):
        """pop game details"""
        rom_name = rom_name.lower()
        #for each child
        for evb in self.fixd.get_children():
            if evb.get_name() in _items:
                lbl = evb.child
                #lbl.set_text('')
                #get list of possible control.ini items via widget name
                for ctrl_item in _ctrls_map[evb.get_name()]:
                    if self.ctrls_ini.has_option(rom_name, ctrl_item):
                        #set label text
                        lbl_text = self.ctrls_ini.get(rom_name, ctrl_item)
                        if evb.get_name() == 'GameName':
                            lbl_text = lbl_text.split('(')[0]
                        elif evb.get_name() == 'NumPlayers':
                            if lbl_text == '1':
                                players = 'player'
                            else:
                                players = 'players'
                            lbl_text = '%s %s' % (lbl_text, players)
                        elif evb.get_name() in ['P1JoyType', 'P2JoyType']:
                            lbl_text = lbl_text.split('+')[0]
                        if len(lbl.get_text()) > 0:
                            lbl_text = '%s / %s' % (lbl.get_text(), lbl_text)
                        lbl.set_text(lbl_text)
                        #break
                    else:
                        if evb.get_name() == 'GameName':
                            lbl.set_text(rom_name)
                        else:
                            lbl.set_text('')
        #done - show window
        self.winCPViewer.show()


def get_viewer_window(win_main):
    """create and return cpviewer class"""
    winCPViewer = None
    #open cpviewer ini
    cpviewer_ini = MameWahIni(os.path.join(win_main.config_dir, 'cpviewer.ini'), 'default')
    if not os.path.exists(cpviewer_ini.get('viewer_layout')):
        print "Error: CPViewer layout file: [%s] does not exist" % (
            cpviewer_ini.get('viewer_layout'))
    else:
        #load viewer window
        winCPViewer = WinCPViewer()
        #set main window
        winCPViewer.WinMain = win_main
        #set ini file
        winCPViewer.cpviewer_ini = cpviewer_ini
        #init
        winCPViewer.init()
    #done
    return winCPViewer

==> wahcade-0.27_2.tbz: share/wahcade/win_cpviewer.py <==
# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        win_cpviewer.py
# Description: cp viewer window
# Copyright (c) 2008   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import os
import sys
import new
import types
import ConfigParser
try:
    import xml.etree.ElementTree as ET # in python >=2.5
except ImportError:
    try:
        import cElementTree as ET # effbot's C module
    except ImportError:
        try:
            import elementtree.ElementTree as ET # effbot's pure Python module
        except ImportError:
            try:
                import lxml.etree as ET # ElementTree API using libxml2
            except ImportError:
                import warnings
                warning.warn("could not import ElementTree "
                             "(http://effbot.org/zone/element-index.htm)")
#gtk
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import pango

#project modules
from constants import *
from wc_common import WahCade
from mamewah_ini import MameWahIni

#constants
_items = [
    'P1JoyUp', 'P1JoyDown', 'P1JoyLeft', 'P1JoyRight',
    'P1B1', 'P1B2', 'P1B3', 'P1B4', 'P1B5', 'P1B6', 'P1B7', 'P1B8',
    'P2JoyUp', 'P2JoyDown', 'P2JoyLeft', 'P2JoyRight',
    'P2B1', 'P2B2', 'P2B3', 'P2B4', 'P2B5', 'P2B6', 'P2B7', 'P2B8',
    'P1JoyType', 'P2JoyType', 'GameName', 'NumPlayers', 'History',
    'Spinner1', 'Spinner2']
_ctrls_map = {
    'P1JoyUp': ['P1_JOYSTICK_UP', 'P1_JOYSTICKLEFT_UP'],
    'P1JoyDown': ['P1_JOYSTICK_DOWN', 'P1_JOYSTICKLEFT_DOWN'],
    'P1JoyLeft': ['P1_JOYSTICK_LEFT', 'P1_JOYSTICKLEFT_LEFT'],
    'P1JoyRight': ['P1_JOYSTICK_RIGHT', 'P1_JOYSTICKLEFT_RIGHT'],
    'P1B1': ['P1_BUTTON1'],
    'P1B2': ['P1_BUTTON2'],
    'P1B3': ['P1_BUTTON3'],
    'P1B4': ['P1_BUTTON4'],
    'P1B5': ['P1_BUTTON5'],
    'P1B6': ['P1_BUTTON6'],
    'P1B7': ['P1_BUTTON7'],
    'P1B8': ['P1_BUTTON8'],
    'P2JoyUp': ['P2_JOYSTICK_UP', 'P1_JOYSTICKRIGHT_UP'],
    'P2JoyDown': ['P2_JOYSTICK_DOWN', 'P1_JOYSTICKRIGHT_DOWN'],
    'P2JoyLeft': ['P2_JOYSTICK_LEFT', 'P1_JOYSTICKRIGHT_LEFT'],
    'P2JoyRight': ['P2_JOYSTICK_RIGHT', 'P1_JOYSTICKRIGHT_RIGHT'],
    'P2B1': ['P2_BUTTON1'],
    'P2B2': ['P2_BUTTON2'],
    'P2B3': ['P2_BUTTON3'],
    'P2B4': ['P2_BUTTON4'],
    'P2B5': ['P2_BUTTON5'],
    'P2B6': ['P2_BUTTON6'],
    'P2B7': ['P2_BUTTON7'],
    'P2B8': ['P2_BUTTON8'],
    'P1JoyType': ['P1Controls'],
    'P2JoyType': ['P2Controls'],
    'GameName': ['gamename'],
    'NumPlayers': ['numPlayers'],
    'History': ['miscDetails'],
    'Spinner1': ['P1_PADDLE', 'P1_PADDLE_EXT'],
    'Spinner2': ['P1_PADDLE_V', 'P1_PADDLE_V_EXT']
}

class WinCPViewer(WahCade):
    """cpviewer based control panel viewer"""
    WinMain = None

    def init(self):
        """setup cp viewer window"""
        #build gui
        self.winCPViewer = gtk.Window()
        self.fixd = gtk.Fixed()
        self.imgBackground = gtk.Image()
        self.fixd.add(self.imgBackground)
        self.winCPViewer.add(self.fixd)
        self.imgBackground.show()
        self.fixd.show()
        self.winCPViewer.set_decorated(False)
        self.winCPViewer.set_transient_for(self.WinMain.winMain)
        self.winCPViewer.set_modal(True)
        self.winCPViewer.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.winCPViewer.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
        #self.winCPViewer.connect('destroy', self.on_winCPViewer_destroy)
        #get keyboard & mouse events
        self.winCPViewer.add_events(
            gtk.gdk.POINTER_MOTION_MASK | 
            gtk.gdk.BUTTON_PRESS_MASK | 
            gtk.gdk.BUTTON_RELEASE_MASK |
            gtk.gdk.KEY_PRESS |
            gtk.gdk.KEY_RELEASE)
        self.winCPViewer.connect('key-press-event', self.WinMain.on_winMain_key_press)
        self.winCPViewer.connect('key-release-event', self.WinMain.on_winMain_key_press)
        self.winCPViewer.connect('motion-notify-event', self.WinMain.on_winMain_key_press)
        #load layout & controls info
        self.ctrls_ini = self.get_controls_ini(self.cpviewer_ini.get('controls_ini_file'))
        self.load_layout(self.cpviewer_ini.get('viewer_layout'))
        #init window
        self.winCPViewer.realize()
        if not self.WinMain.wahcade_ini.getint('show_cursor'):
            self.hide_mouse_cursor(self.winCPViewer)
        if self.WinMain.config_opts.fullscreen and not self.WinMain.config_opts.windowed:
            self.winCPViewer.fullscreen()
        self.app_number = 0

    def on_winMain_delete_event(self, *args):
        """exit window"""
        #hide window
        self.winCPViewer.hide()
        return True

    def _make_label(self, widget_name):
        """create a label (inside an event box)"""
        evb = gtk.EventBox()
        evb.set_name(widget_name)
        lbl = gtk.Label(widget_name)
        lbl.show()
        lbl.set_line_wrap(True)
        lbl.set_text('')
        evb.set_size_request(100, 25)
        evb.set_visible_window(True)
        evb.show()
        evb.add(lbl)
        evb.set_property('visible', False)
        return evb
        
    def load_layout(self, xml_filename):
        """parse cpviewer xml file and create & position labels"""
        #init widgets
        cpv_widgets = {}
        for i, widget_name in enumerate(_items):
            evb = self._make_label(widget_name)
            cpv_widgets[widget_name] = evb
            self.fixd.add(evb)
        #get category info
        for event, ctrl_element in ET.iterparse(xml_filename):
            if ctrl_element.tag in _items:
                #get label / event box
                evb = cpv_widgets[ctrl_element.tag]
                lbl = evb.child
                #font
                fd = ctrl_element.find('FontName').text
                if ctrl_element.find('FontBold').text == 'True':
                    fd += ' Bold'
                if ctrl_element.find('FontItalic').text == 'True':
                    fd += ' Italic'
                fd += ' %s' % (ctrl_element.find("FontSize").text)
                font_desc = pango.FontDescription(fd)
                #colours
                transparent = True
                if ctrl_element.find('Transparent') is not None:
                    transparent = (ctrl_element.find('Transparent').text == 'True')
                fg_col = gtk.gdk.color_parse(self.get_colour(abs(int(ctrl_element.find('ForeColor').text))))
                bg_col = gtk.gdk.color_parse(self.get_colour(abs(int(ctrl_element.find('BackColor').text))))
                lbl.modify_font(font_desc)
                lbl.modify_fg(gtk.STATE_NORMAL, fg_col)
                if transparent:
                    evb.set_visible_window(False)
                else:
                    evb.modify_bg(gtk.STATE_NORMAL, bg_col)
                #visible?
                evb.set_property('visible', ctrl_element.find('Visible').text == 'True')
                #alignment
                if ctrl_element.find('TextAlign').text == 'MiddleLeft':
                    align = 0.0
                    justify = gtk.JUSTIFY_LEFT
                elif ctrl_element.find('TextAlign').text == 'MiddleCenter':
                    align = 0.5
                    justify = gtk.JUSTIFY_CENTER
                elif ctrl_element.find('TextAlign').text == 'MiddleRight':
                    align = 1.0
                    justify = gtk.JUSTIFY_RIGHT
                lbl.set_alignment(align, 0.5)
                lbl.set_property('justify', justify)
                #move & size
                evb.set_size_request(
                    int(ctrl_element.find('Width').text),
                    int(ctrl_element.find('Height').text))
                self.fixd.move(
                    evb,
                    int(ctrl_element.find('Left').text),
                    int(ctrl_element.find('Top').text))
            elif ctrl_element.tag  == 'MainForm':
                #setup background, etc
                main_width = int(ctrl_element.find('Width').text)
                main_height = int(ctrl_element.find('Height').text)
                main_img = self.get_path(ctrl_element.find('BGImage').text)
                if ctrl_element.find('BackColor') is not None:
                    main_bg_col = gtk.gdk.color_parse(self.get_colour(abs(int(ctrl_element.find('BackColor').text))))
                    self.winCPViewer.modify_bg(gtk.STATE_NORMAL, main_bg_col)
                self.winCPViewer.set_size_request(main_width, main_height)
                self.winCPViewer.set_default_size(main_width, main_height)
                self.imgBackground.set_size_request(main_width, main_height)
                if os.path.exists(main_img):
                    self.imgBackground.set_from_file(main_img)
        #done
        ctrl_element.clear()

    def get_controls_ini(self, controls_file):
        """load controls.ini file"""
        ctrls_ini = ConfigParser.SafeConfigParser()
        ctrls_ini.read(controls_file)
        return ctrls_ini

    def display_game_details(self, rom_name):
        """pop game details"""
        rom_name = rom_name.lower()
        #for each child
        for evb in self.fixd.get_children():
            if evb.get_name() in _items:
                lbl = evb.child
                #lbl.set_text('')
                #get list of possible control.ini items via widget name
                for ctrl_item in _ctrls_map[evb.get_name()]:
                    if self.ctrls_ini.has_option(rom_name, ctrl_item):
                        #set label text
                        lbl_text = self.ctrls_ini.get(rom_name, ctrl_item)
                        if evb.get_name() == 'GameName':
                            lbl_text = lbl_text.split('(')[0]
                        elif evb.get_name() == 'NumPlayers':
                            if lbl_text == '1':
                                players = 'player'
                            else:
                                players = 'players'
                            lbl_text = '%s %s' % (lbl_text, players)
                        elif evb.get_name() in ['P1JoyType', 'P2JoyType']:
                            lbl_text = lbl_text.split('+')[0]
                        #if len(lbl.get_text()) > 0:
                        #    lbl_text = '%s / %s' % (lbl.get_text(), lbl_text)
                        lbl.set_text(lbl_text)
                        #break
                    else:
                        if evb.get_name() == 'GameName':
                            lbl.set_text(rom_name)
                        else:
                            lbl.set_text('')
        #done - show window
        self.winCPViewer.show()


def get_viewer_window(win_main):
    """create and return cpviewer class"""
    winCPViewer = None
    #open cpviewer ini
    cpviewer_ini = MameWahIni(os.path.join(win_main.config_dir, 'cpviewer.ini'), 'default')
    if not os.path.exists(cpviewer_ini.get('viewer_layout')):
        print "Error: CPViewer layout file: [%s] does not exist" % (
            cpviewer_ini.get('viewer_layout'))
    else:
        #load viewer window
        winCPViewer = WinCPViewer()
        #set main window
        winCPViewer.WinMain = win_main
        #set ini file
        winCPViewer.cpviewer_ini = cpviewer_ini
        #init
        winCPViewer.init()
    #done
    return winCPViewer
